#include <def21364.h>

//Application definitions.
//The definition for SPIMS is not included in VisualDSP 3.0,
//please contact dsp.support@analog.com for the latest version
//of def21266.h if this project does not build.
#define COMMON_SPI_SETTINGS (SPIEN|SPIMS|SENDZ|CPHASE|CLKPL)
#define COMMON_SPI_DMA_SETTINGS (INTEN|SPIDEN)
#define BITREV_ADDRESS_MASK (0xFFFFFF00)
#define BAUD_RATE_DIVISOR 100
#define SPI_READ            (0x03)  //Read data from memory
#define SPI_RDSR            (0x05)  //Read status register



.section/dm dm_data;
.var curr_command;
.var spi_setting;
.var read_base_addr;
.var flash_curr_page;
.var read_buffer_length;
.var read_buffer_addr;
.var spi_dma_setting;
.var status_register;

.global read_base_addr;
.global read_buffer_length;
.global read_buffer_addr;
.global read_data_bytes;
.global read_status_register;

.section/pm pm_code;


//----------------------------------------
//READ STATUS REGISTER SUBROUTINE (2 Bytes)
//Returns the 8-bit value of the status register.
//Inputs - none
//Outputs- second location of status_register[2],
//         first location is garbage.
//Core sends the command
read_status_register:
r0=COMMON_SPI_SETTINGS|TIMOD2|WL8|MSBF;
dm(spi_setting)=r0;
call setup_spi;

r0=SPI_RDSR;    //Read Status Register command
dm(TXSPI)=r0;

r0=2;
dm(CSPI)=r0;
r0=status_register;
dm(IISPI)=r0;
r0=1;
dm(IMSPI)=r0;
r0=COMMON_SPI_DMA_SETTINGS|SPIRCV;
dm(spi_dma_setting)=r0;

call setup_spi_dma;

dm(SPICTL)=r15; //Disable SPI
dm(SPIFLG)=r15;
dm(SPIBAUD)=r15;

//Pause before continuing
lcntr=1000, do (pc,1) until lce;
nop;

read_status_register.end: rts;

//----------------------------------------
//READ DATA BYTES SUBROUTINE (read_buffer_length 32-bit words)
//Reads the specified number of words from the Flash.
//Inputs - read_base_addr - Starting Flash address
//       - read_buffer_addr - Address of buffer to write
//       - read_buffer_length - Number of 32-bit words to read
//         plus one
//Outputs- Buffer specified by read_buffer_addr will be updated
//         by the specified number of words plus one. The first
//         32-bit word in the buffer is not valid
//Core sends the command
read_data_bytes:
ustat3 = 0;

r0=SPI_READ;
dm(curr_command)=r0;
r0=COMMON_SPI_SETTINGS|TIMOD2|WL32;
dm(spi_setting)=r0;
call setup_spi;

r0=dm(read_base_addr);
dm(flash_curr_page)=r0;

call bit_reverse_command;
r0=dm(curr_command);
dm(TXSPI)=r0;

r0=dm(read_buffer_length);
dm(CSPI)=r0;
r0=dm(read_buffer_addr);
dm(IISPI)=r0;
r0=1;
dm(IMSPI)=r0;
r0=COMMON_SPI_DMA_SETTINGS|SPIRCV;
dm(spi_dma_setting)=r0;

call setup_spi_dma;

dm(SPICTL)=ustat3; //Disable SPI
dm(SPIFLG)=ustat3;
dm(SPIBAUD)=ustat3;
lcntr=1000, do (pc,1) until lce;
nop;

read_data_bytes.end: rts;

//----------------------------------------
//SETUP SPI SUBROUTINE (1 Byte)
//Sets up the SPI for mode specified in spi_setting
//Inputs - spi_setting
//Outputs- none
setup_spi:

r0=0xFE01;
dm(SPIFLG)=r0;

r0=BAUD_RATE_DIVISOR;
dm(SPIBAUD)=r0;

r0=dm(spi_setting);
dm(SPICTL)=r0;

setup_spi.end: rts;

//----------------------------------------
//SETUP SPI DMA SUBROUTINE (1 Byte)
//Sets up the SPI DMA for mode specified in spi_dma_setting
//Inputs - spi_dma_setting
//Outputs- none

setup_spi_dma:
bit set mode1 IRPTEN;
r0=dm(spi_dma_setting);
dm(SPIDMAC)=r0;

nop;
idle;

//Disable DMA
dm(SPIDMAC)=ustat3;
bit clr mode1 IRPTEN;
setup_spi_dma.end: rts;

//----------------------------------------
//BIT REVERSE COMMAND SUBROUTINE (1 Byte)
//For booting the SHARC EX requires that the data in the Flash
//be sent LSB first, but all PROM/FLASH devices are MSB first
//so the data must be programmed into the Flash LSBF, but the
//command word for the flash must be bit reversed for the Flash
//to recognize the command in LSBF mode.
//Inputs - curr_command - current command to send
//       - flash_curr_page - address of the page to access
//Outputs- curr_command
bit_reverse_command:
i6=dm(curr_command);
bitrev(i6,0);
r0=i6;
r0=rot r0 by -24;
i6=dm(flash_curr_page);
bitrev(i6,0);
r1=i6;
r2=BITREV_ADDRESS_MASK;
r1=r1 and r2;
r0=r0 or r1;
dm(curr_command)=r0;
bit_reverse_command.end: rts;